from setuptools import setup, find_packages

setup(
    name='jchm',
    version='1.0.5',
    packages=find_packages(),  # Automatically finds the package directories
    install_requires=[
        # List your package dependencies here
    ],
)
